from cmdbox.app import common
from iinfer import version
from pathlib import Path
import getpass
import logging
import platform
import shutil
import sys
import yaml


class Install(object):
    def __init__(self, logger: logging.Logger, wsl_name: str = None, wsl_user: str = None):
        self.logger = logger
        common.set_debug(self.logger, True)
        self.wsl_name = wsl_name
        self.wsl_user = wsl_user

    def redis(self):
        cmd = f"docker pull ubuntu/redis:latest"
        if platform.system() == 'Windows':
            if self.wsl_name is None:
                return {"warn":f"wsl_name option is required."}
            if self.wsl_user is None:
                return {"warn":f"wsl_user option is required."}
            returncode, _, _cmd = common.cmd(f"wsl -d {self.wsl_name} -u {self.wsl_user} {cmd}", self.logger, slise=-1)
            if returncode != 0:
                self.logger.warning(f"Failed to install redis-server. cmd:{_cmd}")
                return {"error": f"Failed to install redis-server. cmd:{_cmd}"}
            return {"success": f"Success to install redis-server. cmd:{_cmd}"}

        elif platform.system() == 'Linux':
            returncode, _, _cmd = common.cmd(f"{cmd}", self.logger, slise=-1)
            if returncode != 0:
                self.logger.warning(f"Failed to install redis-server. cmd:{_cmd}")
                return {"error": f"Failed to install redis-server. cmd:{_cmd}"}
            return {"success": f"Success to install redis-server. cmd:{_cmd}"}

        else:
            return {"warn":f"Unsupported platform."}

    def server(self, data:Path, install_cmdbox_tgt:str='cmdbox', install_iinfer_tgt:str='iinfer', install_onnx:bool=True,
               install_mmdet:bool=True, install_mmseg:bool=True, install_mmcls:bool=False, install_mmpretrain:bool=True,
               install_insightface=False, install_from:str=None, install_no_python:bool=False, install_compile_python:bool=False,
               install_tag:str=None, install_use_gpu:bool=False,
               tts_engine:str=None, voicevox_ver:str=None, voicevox_whl:str=None):
        """
        iinferが含まれるdockerイメージをインストールします。

        Args:
            data (Path): iinfer-serverのデータディレクトリ
            install_cmdbox_tgt (str): cmdboxのインストール元
            install_iinfer_tgt (str): iinferのインストール元
            install_onnx (bool): onnxをインストールするかどうか
            install_mmdet (bool): mmdetをインストールするかどうか
            install_mmseg (bool): mmsegをインストールするかどうか
            install_mmcls (bool): mmclsをインストールするかどうか
            install_mmpretrain (bool): mmpretrainをインストールするかどうか
            install_insightface (bool): insightfaceをインストールするかどうか
            install_from (str): インストール元dockerイメージ
            install_no_python (bool): pythonをインストールしない
            install_compile_python (bool): pythonをコンパイルしてインストール
            install_tag (str): インストールタグ
            install_use_gpu (bool): GPUを使用するモジュール構成でインストールします。
            tts_engine (str): TTSエンジンの指定
            voicevox_ver (str): VoiceVoxのバージョン
            voicevox_whl (str): VoiceVoxのwhlファイルの名前

        Returns:
            dict: 処理結果
        """
        if platform.system() == 'Windows':
            return {"warn": f"Build server command is Unsupported in windows platform."}
        from importlib.resources import read_text
        user = getpass.getuser()
        install_tag = f"_{install_tag}" if install_tag is not None else ''
        with open('Dockerfile', 'w', encoding='utf-8') as fp:
            text = read_text(f'iinfer.docker', 'Dockerfile')
            # cmdboxのインストール設定
            wheel_cmdbox = Path(install_cmdbox_tgt)
            if wheel_cmdbox.exists() and wheel_cmdbox.suffix == '.whl':
                shutil.copy(wheel_cmdbox, Path('.').resolve() / wheel_cmdbox.name)
                install_cmdbox_tgt = f'/home/{user}/{wheel_cmdbox.name}'
                text = text.replace('#{COPY_CMDBOX}', f'COPY {wheel_cmdbox.name} {install_cmdbox_tgt}')
            else:
                text = text.replace('#{COPY_CMDBOX}', '')
            # iinferのインストール設定
            wheel_iinfer = Path(install_iinfer_tgt)
            if wheel_iinfer.exists() and wheel_iinfer.suffix == '.whl':
                shutil.copy(wheel_iinfer, Path('.').resolve() / wheel_iinfer.name)
                install_iinfer_tgt = f'/home/{user}/{wheel_iinfer.name}'
                text = text.replace('#{COPY_IINFER}', f'COPY {wheel_iinfer.name} {install_iinfer_tgt}')
            else:
                text = text.replace('#{COPY_IINFER}', '')

            start_sh_src = Path(__file__).parent.parent / 'docker' / 'scripts'
            start_sh_tgt = f'scripts'
            shutil.copytree(start_sh_src, start_sh_tgt, dirs_exist_ok=True)
            text = text.replace('#{COPY_IINFER_START}', f'COPY {start_sh_tgt} {start_sh_tgt}')

            install_use_gpu_opt = '--install_use_gpu' if install_use_gpu else ''
            base_image = 'python:3.11.9-slim' #'python:3.8.18-slim'
            if install_use_gpu:
                #base_image = 'nvidia/cuda:11.8.0-cudnn8-runtime-ubuntu22.04'
                #base_image = 'pytorch/pytorch:2.7.0-cuda12.8-cudnn9-runtime'
                #base_image = 'pytorch/pytorch:2.1.0-cuda12.1-cudnn8-runtime'
                base_image = 'pytorch/pytorch:2.4.0-cuda12.1-cudnn9-runtime'
            if install_from is not None and install_from != '':
                base_image = install_from
            text = text.replace('#{FROM}', f'FROM {base_image}')
            text = text.replace('${MKUSER}', user)
            #text = text.replace('#{INSTALL_PYTHON}', f'RUN apt-get update && apt-get install -y python3.8 python3.8-distutils python3-pip python-is-python3' if install_use_gpu else '')
            if install_compile_python:
                install_python = f'RUN apt-get update && apt-get install -y build-essential libbz2-dev libdb-dev libreadline-dev libffi-dev libgdbm-dev liblzma-dev ' + \
                                 f'libncursesw5-dev libsqlite3-dev libssl-dev zlib1g-dev uuid-dev tk-dev wget\n' + \
                                 f'RUN wget https://www.python.org/ftp/python/3.11.11/Python-3.11.11.tar.xz\n' + \
                                 f'RUN tar xJf Python-3.11.11.tar.xz && cd Python-3.11.11 && ./configure && make && make install\n' + \
                                 f'RUN update-alternatives --install /usr/bin/python python /usr/local/bin/python3.11 1'
                                 #f'RUN update-alternatives --install /usr/bin/python python /usr/local/bin/python3.11 1\n' + \
                                 #f'RUN wget https://bootstrap.pypa.io/get-pip.py && python get-pip.py\n'
                text = text.replace('#{INSTALL_PYTHON}', install_python)
            elif not install_no_python:
                # pytorch/pytorch:2.7.0-cuda12.8-cudnn9-runtime の場合python3.11はインストール済み
                text = text.replace('#{INSTALL_PYTHON}', '')
                #text = text.replace('#{INSTALL_PYTHON}', f'RUN apt-get update && apt-get install -y python3.11 python3.11-distutils python3-pip python-is-python3')
                #text = text.replace('#{INSTALL_PYTHON}', f'RUN apt-get update && apt-get install -y python3.11 python3.11-distutils python3.11-dev\n' + \
                #                                         f'RUN update-alternatives --install /usr/bin/python python /usr/bin/python3.11 1\n' + \
                #                                         f'RUN wget https://bootstrap.pypa.io/get-pip.py && python get-pip.py && apt-get install -y python3.11-venv\n')
            else:
                text = text.replace('#{INSTALL_PYTHON}', '')
            install_voicevox = ''
            if tts_engine is not None and tts_engine.lower() == 'voicevox':
                install_voicevox = f'RUN pip install https://github.com/VOICEVOX/voicevox_core/releases/download/{voicevox_ver}/{voicevox_whl}'
                #install_voicevox = f'RUN iinfer -m tts -c install --data /home/{user}/.iinfer --client_only --tts_engine voicevox ' \
                #                 + f'--voicevox_ver {voicevox_ver} --voicevox_os linux --voicevox_arc {voicevox_arc} ' \
                #                 + f'--voicevox_device {"cuda" if install_use_gpu else "cpu"} --voicevox_whl {voicevox_whl} --debug'
            text = text.replace('#{INSTALL_TAG}', install_tag)
            text = text.replace('#{INSTALL_CMDBOX}', install_cmdbox_tgt)
            text = text.replace('#{INSTALL_IINFER}', install_iinfer_tgt)
            text = text.replace('#{INSTALL_ONNX}', f'RUN iinfer -m install -c onnx --data /home/{user}/.iinfer {install_use_gpu_opt} --debug' if install_onnx else '')
            text = text.replace('#{INSTALL_MMDET}', f'RUN iinfer -m install -c mmdet --data /home/{user}/.iinfer {install_use_gpu_opt} --debug' if install_mmdet else '')
            text = text.replace('#{INSTALL_MMSEG}', f'RUN iinfer -m install -c mmseg --data /home/{user}/.iinfer {install_use_gpu_opt} --debug' if install_mmseg else '')
            text = text.replace('#{INSTALL_MMCLS}', f'RUN iinfer -m install -c mmcls --data /home/{user}/.iinfer {install_use_gpu_opt} --debug' if install_mmcls else '')
            text = text.replace('#{INSTALL_MMPRETRAIN}', f'RUN iinfer -m install -c mmpretrain --data /home/{user}/.iinfer {install_use_gpu_opt} --debug' if install_mmpretrain else '')
            text = text.replace('#{INSTALL_INSIGHTFACE}', f'RUN iinfer -m install -c insightface --data /home/{user}/.iinfer {install_use_gpu_opt} --debug' if install_insightface else '')
            text = text.replace('#{INSTALL_VOICEVOX}', install_voicevox)
            fp.write(text)
        docker_compose_path = Path('docker-compose.yml')
        if not docker_compose_path.exists():
            with open(docker_compose_path, 'w', encoding='utf-8') as fp:
                text = read_text(f'iinfer.docker', 'docker-compose.yml')
                fp.write(text)
        with open(f'docker-compose.yml', 'r+', encoding='utf-8') as fp:
            comp = yaml.safe_load(fp)
            services = comp['services']
            common.mkdirs(data)
            services[f'iinfer_server{install_tag}'] = dict(
                image=f'hamacom/iinfer:{version.__version__}{install_tag}',
                container_name=f'iinfer_server{install_tag}',
                environment=dict(
                    TZ='Asia/Tokyo',
                    IINFER_DEBUG='false',
                    REDIS_HOST='${REDIS_HOST:-redis}',
                    REDIS_PORT='${REDIS_PORT:-6379}',
                    REDIS_PASSWORD='${REDIS_PASSWORD:-password}',
                    SVNAME='${SVNAME:-server'+install_tag+'}',
                    LISTEN_PORT='${LISTEN_PORT:-8081}',
                    MCPSV_LISTEN_PORT='${MCPSV_LISTEN_PORT:-8082}',
                    SVCOUNT='${SVCOUNT:-2}',
                ),
                user=user,
                ports=['${LISTEN_PORT:-8081}:${LISTEN_PORT:-8081}', '${MCPSV_LISTEN_PORT:-8082}:${MCPSV_LISTEN_PORT:-8082}'],
                privileged=True,
                restart='always',
                working_dir=f'/home/{user}',
                devices=['/dev/bus/usb:/dev/bus/usb'],
                volumes=[
                    f'{data}:/home/{user}/.iinfer',
                    f'/home/{user}/scripts:/home/{user}/scripts',
                    f'/home/{user}:/home/{user}'
                ],
                command=f'CMD bash /home/{user}/scripts/start.sh'
            )
            if install_use_gpu:
                services[f'iinfer_server{install_tag}']['deploy'] = dict(
                    resources=dict(reservations=dict(devices=[dict(
                        driver='nvidia',
                        count=1,
                        capabilities=['gpu']
                    )]))
                )
            fp.seek(0)
            yaml.dump(comp, fp)
        cmd = f'docker build -t hamacom/iinfer:{version.__version__}{install_tag} -f Dockerfile .'

        if platform.system() == 'Linux':
            returncode, _, _cmd = common.cmd(f"{cmd}", self.logger, slise=-1)
            #os.remove('Dockerfile')
            if returncode != 0:
                self.logger.warning(f"Failed to install iinfer-server. cmd:{_cmd}")
                return {"error": f"Failed to install iinfer-server. cmd:{_cmd}"}
            return {"success": f"Success to install iinfer-server. and docker-compose.yml is copied. cmd:{_cmd}"}

        else:
            return {"warn":f"Unsupported platform."}

    def onnx(self, install_use_gpu:bool=False):
        returncode, _, _cmd = common.cmd('pip install onnxruntime', logger=self.logger, slise=-1)
        if returncode != 0:
            self.logger.warning(f"Failed to install onnxruntime. cmd:{_cmd}")
            return {"error": f"Failed to install onnxruntime. cmd:{_cmd}"}
        if install_use_gpu:
            returncode, _, _cmd = common.cmd('pip install onnxruntime-gpu', logger=self.logger, slise=-1)
            if returncode != 0:
                self.logger.warning(f"Failed to install onnxruntime-gpu. cmd:{_cmd}")
                return {"error": f"Failed to install onnxruntime-gpu. cmd:{_cmd}"}
        return {"success": f"Success to install onnxruntime."}

    def insightface(self, data_dir: Path, install_use_gpu:bool=False):
        returncode, _, _cmd = common.cmd('pip install cython', logger=self.logger, slise=-1)
        if returncode != 0:
            self.logger.warning(f"Failed to install cython. cmd:{_cmd}")
            return {"error": f"Failed to install cython. cmd:{_cmd}"}
        ret = self.onnx(install_use_gpu)
        if "error" in ret: return ret
        returncode, _, _cmd = common.cmd('python -m pip install --upgrade pip setuptools', logger=self.logger, slise=-1)
        if returncode != 0:
            self.logger.warning(f"Failed to install setuptools. cmd:{_cmd}")
            return {"error": f"Failed to install setuptools. cmd:{_cmd}"}
        returncode, _, _cmd = common.cmd('pip install insightface', logger=self.logger, slise=-1)
        if returncode != 0:
            self.logger.warning(f"Failed to install insightface. cmd:{_cmd}")
            return {"error": f"Failed to install insightface. cmd:{_cmd}"}
        return {"success": f"Success to install insightface."}

    def _numpy(self):
        if sys.version_info[0] >= 3 and sys.version_info[1] >= 10:
            returncode, _, _cmd = common.cmd('pip install numpy==1.26.4', logger=self.logger, slise=-1)
        else:
            returncode, _, _cmd = common.cmd('pip install numpy==1.24.1', logger=self.logger, slise=-1)
        if returncode != 0:
            self.logger.warning(f"Failed to install numpy. cmd:{_cmd}")
            return {"error": f"Failed to install numpy. cmd:{_cmd}"}
        return {"success": f"Success to install numpy. cmd:{_cmd}"}

    """
    def _torch(self, install_use_gpu:bool=False):
        if install_use_gpu:
            if sys.version_info[0] >= 3 and sys.version_info[1] >= 10:
                returncode, _, _cmd = common.cmd('pip install numpy==1.26.3 torch==2.1.0 torchvision torchaudio --index-url https://download.pytorch.org/whl/cu118', logger=self.logger, slise=-1)
            else:
                returncode, _, _cmd = common.cmd('pip install numpy==1.24.1 torch==2.1.0 torchvision torchaudio --index-url https://download.pytorch.org/whl/cu118', logger=self.logger, slise=-1)
        else:
            if sys.version_info[0] >= 3 and sys.version_info[1] >= 10:
                returncode, _, _cmd = common.cmd('pip install numpy==1.26.3 torch==2.1.0 torchvision torchaudio', logger=self.logger, slise=-1)
            else:
                returncode, _, _cmd = common.cmd('pip install numpy==1.24.1 torch==2.1.0 torchvision torchaudio', logger=self.logger, slise=-1)
        if returncode != 0:
            self.logger.warning(f"Failed to install torch torchvision torchaudio. cmd:{_cmd}")
            return {"error": f"Failed to install torch torchvision torchaudio. cmd:{_cmd}"}
        return {"success": f"Success to install torch torchvision torchaudio. cmd:{_cmd}"}
    """
    def _openmin(self, install_use_gpu:bool=False):
        returncode, _, _cmd = common.cmd('pip install openmim', logger=self.logger, slise=-1)
        if returncode != 0:
            self.logger.warning(f"Failed to install openmim. cmd:{_cmd}")
            return {"error": f"Failed to install openmim. cmd:{_cmd}"}
        return {"success": f"Success to install openmim. cmd:{_cmd}"}

    def _mmcv(self, install_use_gpu:bool=False):
        msg = self._numpy()
        if "success" not in msg: return msg
        if install_use_gpu:
            #returncode, _, _cmd = common.cmd('pip install mmcv==2.1.0 -f https://download.openmmlab.com/mmcv/dist/cu121/torch2.1/index.html', logger=self.logger, slise=-1)
            returncode, _, _cmd = common.cmd('pip install mmcv==2.2.0 -f https://download.openmmlab.com/mmcv/dist/cu121/torch2.4/index.html', logger=self.logger, slise=-1)
            #returncode, _, _cmd = common.cmd('mim install mmcv==2.2.0', logger=self.logger, slise=-1)
            #returncode, _, _cmd = common.cmd('pip install mmcv==2.1.0 -f https://download.openmmlab.com/mmcv/dist/cu118/torch2.1/index.html', logger=self.logger, slise=-1)
            if returncode != 0:
                self.logger.warning(f"Failed to install mmcv. cmd:{_cmd}")
                return {"error": f"Failed to install mmcv. cmd:{_cmd}"}
        else:
            returncode, _, _cmd = common.cmd('mim install mmcv==2.2.0', logger=self.logger, slise=-1)
            #returncode, _, _cmd = common.cmd('mim install mmcv==2.1.0', logger=self.logger, slise=-1)
            #returncode, _, _cmd = common.cmd('mim install mmcv>=2.0.0', logger=self.logger, slise=-1)
        if returncode != 0:
            self.logger.warning(f"Failed to install mmcv. cmd:{_cmd}")
            return {"error": f"Failed to install mmcv. cmd:{_cmd}"}
        return {"success": f"Success to install mmcv. cmd:{_cmd}"}

    def _transformers(self, install_use_gpu:bool=False):
        returncode, _, _cmd = common.cmd('pip install accelerate transformers bitsandbytes sentence-transformers', logger=self.logger, slise=-1)
        if returncode != 0:
            self.logger.warning(f"Failed to install accelerate transformers bitsandbytes sentence-transformers. cmd:{_cmd}")
            return {"error": f"Failed to install accelerate transformers bitsandbytes sentence-transformers. cmd:{_cmd}"}
        return {"success": f"Success to install accelerate transformers bitsandbytes sentence-transformers. cmd:{_cmd}"}

    def mmdet(self, data_dir:Path, install_use_gpu:bool=False):
        returncode, _, _cmd = common.cmd(f'git clone https://github.com/open-mmlab/mmdetection.git', logger=self.logger, slise=-1)
        if returncode != 0:
            self.logger.warning(f"Failed to git clone mmdetection. Delete mmdetection as it probably already exists. cmd:{_cmd}")
            return {"error": f"Failed to git clone mmdetection. Delete mmdetection as it probably already exists. cmd:{_cmd}"}
        srcdir = Path('.') / 'mmdetection'
        shutil.copytree(srcdir, data_dir / 'mmdetection', dirs_exist_ok=True, ignore=shutil.ignore_patterns('.git'))
        shutil.rmtree(srcdir, ignore_errors=True)

        #ret = self._torch(install_use_gpu)
        #if "error" in ret: return ret
        ret = self._openmin(install_use_gpu)
        if "error" in ret: return ret
        ret = self._mmcv(install_use_gpu)
        if "error" in ret: return ret
        msg = self._numpy()
        if "success" not in msg: return msg

        ret, _, _cmd = common.cmd('mim install mmdet', logger=self.logger, slise=-1)
        if ret != 0:
            self.logger.warning(f"Failed to install mmdet. cmd:{_cmd}")
            return {"error": f"Failed to install mmdet. cmd:{_cmd}"}

        ret, _, _cmd = common.cmd(f'sed -i "s/mmcv_maximum_version = \'2.2.0\'/mmcv_maximum_version = \'2.2.1\'/" /opt/conda/lib/python3.11/site-packages/mmdet/__init__.py', logger=self.logger, slise=-1)
        if ret != 0:
            self.logger.warning(f"Failed to install mmdet. cmd:{_cmd}")
            return {"error": f"Failed to install mmdet. cmd:{_cmd}"}

        if srcdir.exists():
            return {"success": f"Please remove '{srcdir / 'mmdetection'}' manually. cmd:{_cmd}"}
        return {"success": f"Success to install mmdet. cmd:{_cmd}"}

    def mmseg(self, data_dir:Path, install_use_gpu:bool=False):
        returncode, _, _cmd = common.cmd(f'git clone -b main https://github.com/open-mmlab/mmsegmentation.git', logger=self.logger, slise=-1)
        if returncode != 0:
            self.logger.warning(f"Failed to git clone mmsegmentation. Delete mmsegmentation as it probably already exists. cmd:{_cmd}")
            return {"error": f"Failed to git clone mmsegmentation. Delete mmsegmentation as it probably already exists. cmd:{_cmd}"}
        srcdir = Path('.') / 'mmsegmentation'
        shutil.copytree(srcdir, data_dir / 'mmsegmentation', dirs_exist_ok=True, ignore=shutil.ignore_patterns('.git'))
        shutil.rmtree(srcdir, ignore_errors=True)

        #ret = self._torch(install_use_gpu)
        #if "error" in ret: return ret
        ret = self._openmin(install_use_gpu)
        if "error" in ret: return ret
        ret = self._mmcv(install_use_gpu)
        if "error" in ret: return ret
        msg = self._numpy()
        if "success" not in msg: return msg

        ret, _, _cmd = common.cmd('mim install mmsegmentation', logger=self.logger, slise=-1)
        if ret != 0:
            self.logger.warning(f"Failed to install mmsegmentation. cmd:{_cmd}")
            return {"error": f"Failed to install mmsegmentation. cmd:{_cmd}"}

        ret, _, _cmd = common.cmd('pip install ftfy', logger=self.logger, slise=-1)
        if ret != 0:
            self.logger.warning(f"Failed to install ftfy. cmd:{_cmd}")
            return {"error": f"Failed to install ftfy. cmd:{_cmd}"}

        ret, _, _cmd = common.cmd('pip install regex', logger=self.logger, slise=-1)
        if ret != 0:
            self.logger.warning(f"Failed to install regex. cmd:{_cmd}")
            return {"error": f"Failed to install regex. cmd:{_cmd}"}

        if srcdir.exists():
            return {"success": f"Please remove '{srcdir / 'mmsegmentation'}' manually."}
        return {"success": f"Success to install mmsegmentation."}

    def mmcls(self, data_dir:Path, install_use_gpu:bool=False):

        #ret = self._torch(install_use_gpu)
        #if "error" in ret: return ret
        ret = self._openmin(install_use_gpu)
        if "error" in ret: return ret
        ret = self._mmcv(install_use_gpu)
        if "error" in ret: return ret
        msg = self._numpy()
        if "success" not in msg: return msg

        ret, _, _cmd = common.cmd('mim install mmcls', logger=self.logger, slise=-1)
        if ret != 0:
            self.logger.warning(f"Failed to install mmcls. cmd:{_cmd}")
            return {"error": f"Failed to install mmcls. cmd:{_cmd}"}

        return {"success": f"Success to install mmcls. cmd:{_cmd}"}

    def mmpretrain(self, data_dir:Path, install_use_gpu:bool=False):
        returncode, _, _cmd = common.cmd(f'git clone https://github.com/open-mmlab/mmpretrain.git', logger=self.logger, slise=-1)
        if returncode != 0:
            self.logger.warning(f"Failed to git clone mmpretrain. Delete mmpretrain as it probably already exists. cmd:{_cmd}")
            return {"error": f"Failed to git clone mmpretrain. Delete mmpretrain as it probably already exists. cmd:{_cmd}"}
        srcdir = Path('.') / 'mmpretrain'
        shutil.copytree(srcdir, data_dir / 'mmpretrain', dirs_exist_ok=True, ignore=shutil.ignore_patterns('.git'))
        shutil.rmtree(srcdir, ignore_errors=True)

        #ret = self._torch(install_use_gpu)
        #if "error" in ret: return ret
        ret = self._openmin(install_use_gpu)
        if "error" in ret: return ret
        ret = self._mmcv(install_use_gpu)
        if "error" in ret: return ret
        msg = self._numpy()
        if "success" not in msg: return msg

        ret, _, _cmd = common.cmd('mim install mmpretrain', logger=self.logger, slise=-1)
        if ret != 0:
            self.logger.warning(f"Failed to install mmpretrain. cmd:{_cmd}")
            return {"error": f"Failed to install mmpretrain. cmd:{_cmd}"}

        if srcdir.exists():
            return {"success": f"Please remove '{srcdir}' manually. cmd:{_cmd}"}
        return {"success": f"Success to install mmpretrain. cmd:{_cmd}"}



from cmdbox import version
from cmdbox.app import common, feature
from cmdbox.app.options import Options
from iinfer.app import install
from pathlib import Path
from typing import Dict, Any, Tuple, Union, List
import argparse
import logging


class InstallServer(feature.UnsupportEdgeFeature):
    def get_mode(self) -> Union[str, List[str]]:
        """
        この機能のモードを返します

        Returns:
            Union[str, List[str]]: モード
        """
        return 'install'

    def get_cmd(self):
        """
        この機能のコマンドを返します

        Returns:
            str: コマンド
        """
        return 'server'

    def get_option(self):
        """
        この機能のオプションを返します

        Returns:
            Dict[str, Any]: オプション
        """
        return dict(
            use_redis=self.USE_REDIS_FALSE, nouse_webmode=True,
            description_ja="`推論サーバー` のdockerイメージを `build` します。`build` が成功すると、実行時ディレクトリに `docker-compose.yml` ファイルが生成されます。",
            description_en="`Build` the docker image of the `inference server`. If the `build` is successful, a `docker-compose.yml` file is generated in the execution directory.",
            choice=[
                dict(opt="data", type=Options.T_DIR, default=self.default_data, required=False, multi=False, hide=False, choice=None,
                     description_ja=f"省略した時は `$HONE/.{self.ver.__appid__}` を使用します。",
                     description_en=f"When omitted, `$HONE/.{self.ver.__appid__}` is used."),
                dict(opt="install_cmdbox", type=Options.T_STR, default='cmdbox', required=False, multi=False, hide=True, choice=None,
                     description_ja=f"省略した時は `cmdbox` を使用します。 `cmdbox=={version.__version__}` といった指定も可能です。",
                     description_en=f"When omitted, `cmdbox` is used. You can also specify `cmdbox=={version.__version__}`.",
                     test_false={"win":"cmdbox"}),
                dict(opt="install_iinfer", type=Options.T_STR, default='iinfer', required=False, multi=False, hide=True, choice=None,
                     description_ja=f"省略した時は `iinfer` を使用します。 `iinfer=={self.ver.__version__}` といった指定も可能です。",
                     description_en=f"When omitted, `iinfer` is used. You can also specify `iinfer=={self.ver.__version__}`.",
                     test_false={"win":"iinfer"}),
                dict(opt="install_onnx", type=Options.T_BOOL, default=False, required=False, multi=False, hide=True, choice=[True, False],
                     description_ja="dockerイメージ内に `onnxruntime` をインストールします。",
                     description_en="Install `onnxruntime` in the docker image."),
                dict(opt="install_mmdet", type=Options.T_BOOL, default=False, required=False, multi=False, hide=True, choice=[True, False],
                     description_ja="dockerイメージ内に `mmdetection` をインストールします。",
                     description_en="Install `mmdetection` in the docker image."),
                dict(opt="install_mmseg", type=Options.T_BOOL, default=False, required=False, multi=False, hide=True, choice=[True, False],
                     description_ja="dockerイメージ内に `mmsegmentation` をインストールします。",
                     description_en="Install `mmsegmentation` in the docker image."),
                dict(opt="install_mmcls", type=Options.T_BOOL, default=False, required=False, multi=False, hide=True, choice=[True, False],
                     description_ja="dockerイメージ内に `mmclassification` をインストールします。",
                     description_en="Install `mmclassification` in the docker image."),
                dict(opt="install_mmpretrain", type=Options.T_BOOL, default=False, required=False, multi=False, hide=True, choice=[True, False],
                     description_ja="dockerイメージ内に `mmpretrain` をインストールします。",
                     description_en="Install `mmpretrain` in the docker image."),
                dict(opt="install_insightface", type=Options.T_BOOL, default=False, required=False, multi=False, hide=True, choice=[True, False],
                     description_ja="dockerイメージ内に `insightface` をインストールします。",
                     description_en="Install `insightface` in the docker image."),
                dict(opt="install_from", type=Options.T_STR, default=None, required=False, multi=False, hide=False, choice=None,
                     description_ja="作成するdockerイメージの元となるFROMイメージを指定します。",
                     description_en="Specify the FROM image that will be the source of the docker image to be created."),
                dict(opt="install_no_python", type=Options.T_BOOL, default=False, required=False, multi=False, hide=False, choice=[True, False],
                     description_ja="pythonのインストールを行わないようにします。",
                     description_en="Do not install python."),
                dict(opt="install_compile_python", type=Options.T_BOOL, default=False, required=False, multi=False, hide=False, choice=[True, False],
                     description_ja="python3をコンパイルしてインストールします。install_no_pythonが指定されるとそちらを優先します。",
                     description_en="Compile and install python3; if install_no_python is specified, it is preferred."),
                dict(opt="install_tag", type=Options.T_STR, default=None, required=False, multi=False, hide=False, choice=None,
                     description_ja="指定すると作成するdockerイメージのタグ名に追記出来ます。",
                     description_en="If specified, you can add to the tag name of the docker image to create."),
                dict(opt="install_use_gpu", type=Options.T_BOOL, default=False, required=False, multi=False, hide=False, choice=[True, False],
                     description_ja="GPUを使用するモジュール構成でインストールします。",
                     description_en="Install with a module configuration that uses the GPU."),
                dict(opt="tts_engine", type=Options.T_STR, default="voicevox", required=True, multi=False, hide=False,
                     choice=["", "voicevox"],
                     choice_show=dict(voicevox=["voicevox_ver", "voicevox_os", "voicevox_arc", "voicevox_device", "voicevox_whl"]),
                     description_ja="使用するTTSエンジンを指定します。",
                     description_en="Specify the TTS engine to use."),
                dict(opt="voicevox_ver", type=Options.T_STR, default='0.16.3', required=False, multi=False, hide=False,
                     choice=['', '0.16.3'],
                     choice_edit=True,
                     description_ja="使用するVOICEVOXのバージョンを指定します。",
                     description_en="Specify the version of VOICEVOX to use."),
                dict(opt="voicevox_whl", type=Options.T_STR, default='voicevox_core-0.16.3-cp310-abi3-win_amd64.whl', required=False, multi=False, hide=False,
                     choice=['',
                             'voicevox_core-0.16.3-cp310-abi3-win32.whl',
                             'voicevox_core-0.16.3-cp310-abi3-win_amd64.whl',
                             'voicevox_core-0.16.3-cp310-abi3-macosx_10_12_x86_64.whl',
                             'voicevox_core-0.16.3-cp310-abi3-macosx_11_0_arm64.whl',
                             'voicevox_core-0.16.3-cp310-abi3-manylinux_2_34_aarch64.whl',
                             'voicevox_core-0.16.3-cp310-abi3-manylinux_2_34_x86_64.whl',
                             ],
                     choice_edit=True,
                     description_ja="使用するVOICEVOXのホイールファイルを指定します。",
                     description_en="Specify the VOICEVOX wheel file to use."),
                dict(opt="output_json", short="o", type=Options.T_FILE, default=None, required=False, multi=False, hide=True, choice=None, fileio="out",
                     description_ja="処理結果jsonの保存先ファイルを指定。",
                     description_en="Specify the destination file for saving the processing result json."),
                dict(opt="output_json_append", short="a", type=Options.T_BOOL, default=False, required=False, multi=False, hide=True, choice=[True, False],
                     description_ja="処理結果jsonファイルを追記保存します。",
                     description_en="Save the processing result json file by appending."),
                dict(opt="stdout_log", type=Options.T_BOOL, default=True, required=False, multi=False, hide=True, choice=[True, False],
                     description_ja="GUIモードでのみ使用可能です。コマンド実行時の標準出力をConsole logに出力します。",
                     description_en="Available only in GUI mode. Outputs standard output during command execution to Console log."),
                dict(opt="capture_stdout", type=Options.T_BOOL, default=True, required=False, multi=False, hide=True, choice=[True, False],
                     description_ja="GUIモードでのみ使用可能です。コマンド実行時の標準出力をキャプチャーし、実行結果画面に表示します。",
                     description_en="Available only in GUI mode. Captures standard output during command execution and displays it on the execution result screen."),
                dict(opt="capture_maxsize", type=Options.T_INT, default=self.DEFAULT_CAPTURE_MAXSIZE, required=False, multi=False, hide=True, choice=None,
                     description_ja="GUIモードでのみ使用可能です。コマンド実行時の標準出力の最大キャプチャーサイズを指定します。",
                     description_en="Available only in GUI mode. Specifies the maximum capture size of standard output when executing commands."),
            ]
        )

    def apprun(self, logger:logging.Logger, args:argparse.Namespace, tm:float, pf:List[Dict[str, float]]=[]) -> Tuple[int, Dict[str, Any], Any]:
        """
        この機能の実行を行います

        Args:
            logger (logging.Logger): ロガー
            args (argparse.Namespace): 引数
            tm (float): 実行開始時間
            pf (List[Dict[str, float]]): 呼出元のパフォーマンス情報

        Returns:
            Tuple[int, Dict[str, Any], Any]: 終了コード, 結果, オブジェクト
        """
        inst = install.Install(logger=logger, wsl_name=args.wsl_name, wsl_user=args.wsl_user)

        if args.data is None:
            msg = {"warn":f"Please specify the --data option."}
            common.print_format(msg, args.format, tm, args.output_json, args.output_json_append, pf=pf)
            return self.RESP_WARN, msg
        ret = inst.server(Path(args.data),
                          install_cmdbox_tgt=args.install_cmdbox,
                          install_iinfer_tgt=args.install_iinfer,
                          install_onnx=args.install_onnx,
                          install_mmdet=args.install_mmdet,
                          install_mmseg=args.install_mmseg,
                          install_mmcls=args.install_mmcls,
                          install_mmpretrain=args.install_mmpretrain,
                          install_insightface=args.install_insightface,
                          install_from=args.install_from,
                          install_no_python=args.install_no_python,
                          install_compile_python=args.install_compile_python,
                          install_tag=args.install_tag, install_use_gpu=args.install_use_gpu,
                          tts_engine=args.tts_engine, voicevox_ver=args.voicevox_ver, voicevox_whl=args.voicevox_whl)
        common.print_format(ret, args.format, tm, args.output_json, args.output_json_append, pf=pf)

        if 'success' not in ret:
            return self.RESP_WARN, ret, inst
        return self.RESP_SUCCESS, ret, inst

    def audited_by(self, logger:logging.Logger, args:argparse.Namespace) -> bool:
        """
        この機能が監査ログを記録する対象かどうかを返します

        Returns:
            logger (logging.Logger): ロガー
            args (argparse.Namespace): 引数
            bool: 監査ログを記録する場合はTrue
        """
        return False



import datetime

dt_now = datetime.datetime(2025, 12, 16)
days_ago = (datetime.datetime.now() - dt_now).days
__appid__ = 'iinfer'
__title__ = 'iinfer (Image Inference Application)'
__version__ = '0.12.0.3'
__copyright__ = f'Copyright © 2023-{dt_now.strftime("%Y")} hamacom2004jp'
__pypiurl__ = 'https://pypi.org/project/iinfer/'
__srcurl__ = 'https://github.com/hamacom2004jp/iinfer'
__docurl__ = 'https://hamacom2004jp.github.io/iinfer/index.html'
# https://patorjk.com/software/taag/#p=display&f=ANSI%20Shadow&t=iinfer
__logo__ = '''
██╗██╗███╗   ██╗███████╗███████╗██████╗ 
██║██║████╗  ██║██╔════╝██╔════╝██╔══██╗
██║██║██╔██╗ ██║█████╗  █████╗  ██████╔╝
██║██║██║╚██╗██║██╔══╝  ██╔══╝  ██╔══██╗
██║██║██║ ╚████║██║     ███████╗██║  ██║
╚═╝╚═╝╚═╝  ╚═══╝╚═╝     ╚══════╝╚═╝  ╚═╝'''
__description__ = f'{__title__} {__version__}\n\n' + \
                  f'{__copyright__}\n' + \
                  (f'Build Date: {dt_now.strftime("%Y-%m-%d")}\n' if days_ago < 60 else '') + \
                  f'Web Site: PyPi <{__pypiurl__}>\n' + \
                  f'Web Site: SorceCode <{__srcurl__}>\n' + \
                  f'Web Site: Document <{__docurl__}>\n' + \
                  f'License: MIT License <https://opensource.org/license/mit/>\n' + \
                  f'This is free software: you are free to change and redistribute it.\n' + \
                  f'There is NO WARRANTY, to the extent permitted by law.'
__all__ = ['__logo__', '__title__', '__version__', '__copyright__', '__srcurl__', '__docurl__']


from pathlib import Path
from PIL import Image
from typing import List, Tuple, Union, Any
from iinfer.app import common as cmn
from iinfer.app.predicts import onnx_det_YoloV3
import logging
import numpy as np


SITE = 'https://github.com/onnx/models/tree/main/validated/vision/object_detection_segmentation/tiny-yolov3'
IMAGE_WIDTH = 416
IMAGE_HEIGHT = 416
REQUIREd_MODEL_CONF = False
REQUIREd_MODEL_WEIGHT = True

class OnnxDetTinyYoloV3(onnx_det_YoloV3.OnnxDetYoloV3):
    def __init__(self, logger:logging.Logger) -> None:
        super().__init__(logger)

    def predict(self, model, img_width:int, img_height:int, input_data:Union[Image.Image, str], labels:List[str]=None, colors:List[Tuple[int]]=None, nodraw:bool=False):
        """
        予測を行う関数です。
        predictコマンドやcaptureコマンド実行時に呼び出されます。
        引数のinput_dataが画像の場合RGBですので、戻り値の出力画像もRGBにしてください。
        戻り値の推論結果のdictは、通常推論結果項目ごとに値(list)を設定します。

        Args:
            model: 推論セッション
            img_width (int): モデルのINPUTサイズ（input_dataが画像の場合は、画像の幅）
            img_height (int): モデルのINPUTサイズ（input_dataが画像の場合は、画像の高さ）
            input_data (Image | str): 推論するデータ（画像の場合RGB配列であること）
            labels (List[str], optional): クラスラベルのリスト. Defaults to None.
            colors (List[Tuple[int]], optional): ボックスの色のリスト. Defaults to None.
            nodraw (bool, optional): 描画フラグ. Defaults to False.

        Returns:
            Tuple[Dict[str, Any], Image]: 予測結果と出力画像(RGB)のタプル
        """
        image_data, image_size, image_obj = self.preprocess_img(input_data, img_width, img_height)

        input_name = model.get_inputs()[0].name           # 'image'
        input_name_img_shape = model.get_inputs()[1].name # 'image_shape'
        output_name_boxes = model.get_outputs()[0].name   # 'boxes'
        output_name_scores = model.get_outputs()[1].name  # 'scores'
        output_name_indices = model.get_outputs()[2].name # 'indices'

        outputs_index = model.run([output_name_boxes, output_name_scores, output_name_indices],
                                    {input_name: image_data, input_name_img_shape: image_size})

        output_boxes = outputs_index[0]
        output_scores = outputs_index[1]
        output_indices = outputs_index[2]

        out_boxes, out_scores, out_classes = [], [], []
        for idx_ in output_indices[0]:
            out_classes.append(idx_[1])
            out_scores.append(output_scores[tuple(idx_)])
            idx_1 = (idx_[0], idx_[2])
            out_boxes.append(output_boxes[idx_1])

        ids = [i for i in range(len(out_boxes))]
        output_image, output_labels = cmn.draw_boxes(image_obj, out_boxes, out_scores, out_classes, ids=ids, labels=labels, colors=colors, nodraw=nodraw)

        return dict(output_ids=ids, output_scores=out_scores, output_classes=out_classes, output_labels=output_labels, output_boxes=out_boxes), output_image



from cmdbox.app import common, feature
from cmdbox.app.web import Web
from fastapi import FastAPI, Request, Response, HTTPException
from fastapi.responses import StreamingResponse
from starlette.datastructures import UploadFile
import logging
import io
import time


class PubImgProxy(feature.WebFeature):
    def route(self, web:Web, app:FastAPI) -> None:
        """
        webモードのルーティングを設定します

        Args:
            web (Web): Webオブジェクト
            app (FastAPI): FastAPIオブジェクト
        """
        @app.post('/webcap/pub_img/{port:int}')
        async def pub_img_proxy(req:Request, res:Response, port:int):
            """
            webcap画面から送信されてくる画像を、webcapプロセスに送信(プロキシ)する
            webcapプロセスからのレスポンスをそのまま返却する
            webcapプロセスが起動していない場合は502エラーを返却する

            Args:
                req (Request): リクエスト
                res (Response): レスポンス
                port (int): webcapプロセスのポート番号
            """
            signin = web.signin.check_signin(req, res)
            if signin is not None:
                raise HTTPException(status_code=401, detail=self.DEFAULT_401_MESSAGE)
            if web.logger.level == logging.DEBUG:
                web.logger.debug(f"web.pub_img_proxy: target=http://localhost:{port}/webcap/pub_img, header={req.headers}")
            if not req.headers.get('content-type').startswith('multipart/form-data'):
                raise HTTPException(status_code=400, detail='Expected multipart request.')
            try:
                tm = time.perf_counter()
                form = await req.form()
                files = [['files', (fn, await form[fn].read(), form[fn].content_type)] for fn in form.keys() if isinstance(form[fn], UploadFile)]
                responce = web.webcap_client.post(f'http://localhost:{port}/webcap/pub_img', files=files)
                content = responce.content
                if web.logger.level == logging.DEBUG:
                    output_str = common.to_str(content.decode("utf-8"), slise=100)
                    web.logger.debug(f"web.pub_img_proxy: target=http://localhost:{port}/webcap/pub_img, res_status={responce.status_code}, tm={time.perf_counter()-tm:.3f}s")
                responce.headers.pop('server', None)
                return StreamingResponse(io.BytesIO(content), status_code=responce.status_code,
                                         headers=responce.headers, media_type=responce.headers.get('content-type'))
            except Exception as e:
                raise HTTPException(status_code=502, detail=f'web.start.pub_img_proxy: pub_img_proxy closed. {e}') from e



from iinfer.app import injection
from PIL import Image
from typing import Dict, Any
import logging


class BeforeGrayimgInjection(injection.BeforeInjection):
    """
    このクラスは推論を実行する前処理のインジェクションクラスです。
    """
    def __init__(self, config:Dict[str,Any], logger:logging.Logger):
        """
        このクラスのインスタンスを初期化します。
        継承時は、このコンストラクタを呼び出すようにしてください。
            super().__init__(logger)
        Args:
            config (Dict[str,Any]): 設定
            logger (logging.Logger): ロガー
        """
        super().__init__(config, logger)

    def action(self, reskey:str, name:str, image:Image.Image, session:Dict[str, Any]) -> Image.Image:
        """
        このメソッドは推論を実行する前処理を実行します。
        Args:
            reskey (str): レスポンスキー
            name (str): モデル名
            image (Image.Image): 推論する画像データ
            session (Dict[str, Any]): 推論セッション。次の項目が含まれます。
            ・session: app.predict.Predict#create_session() で作成されたセッション
            ・model_img_width: モデルの入力画像の幅
            ・model_img_height: モデルの入力画像の高さ
            ・predict_obj: app.predict.Predict インスタンス
            ・labels: クラスラベルのリスト
            ・colors: ボックスの色のリスト
            ・tracker: use_trackがTrueの場合、トラッカーのインスタンス
        Returns:
            Image.Image: 前処理後の画像データ
        """
        image = image.convert('L')
        image = image.convert('RGB')
        return image



from cmdbox.app import common, feature
from cmdbox.app.options import Options
from iinfer.app import redis
from typing import Dict, Any, Tuple, Union, List
import argparse
import logging


class RedisDockerStop(feature.UnsupportEdgeFeature):
    def get_mode(self) -> Union[str, List[str]]:
        """
        この機能のモードを返します

        Returns:
            Union[str, List[str]]: モード
        """
        return 'redis'

    def get_cmd(self):
        """
        この機能のコマンドを返します

        Returns:
            str: コマンド
        """
        return 'docker_stop'
    
    def get_option(self):
        """
        この機能のオプションを返します

        Returns:
            Dict[str, Any]: オプション
        """
        return dict(
            use_redis=self.USE_REDIS_FALSE, nouse_webmode=True,
            description_ja="installモードで `iinfer -m install -c server` を実行している場合は、 `docker-compose down` を使用してください。",
            description_en="If you are running `iinfer -m install -c server` in install mode, use `docker-compose down`.",
            choice=[
                dict(opt="wsl_name", type=Options.T_STR, default=None, required=False, multi=False, hide=True, choice=None,
                     description_ja="Windowsの場合はWSLのディストリビューションの名前を指定します。",
                     description_en="For Windows, specify the name of the WSL distribution."),
                dict(opt="wsl_user", type=Options.T_STR, default="ubuntu", required=False, multi=False, hide=True, choice=None,
                     description_ja="Windowsの場合はWSL内のユーザー名を指定します。",
                     description_en="For Windows, specify the user name in WSL."),
                dict(opt="output_json", short="o", type=Options.T_FILE, default=None, required=False, multi=False, hide=True, choice=None, fileio="out",
                     description_ja="処理結果jsonの保存先ファイルを指定。",
                     description_en="Specify the destination file for saving the processing result json."),
                dict(opt="output_json_append", short="a", type=Options.T_BOOL, default=False, required=False, multi=False, hide=True, choice=[True, False],
                     description_ja="処理結果jsonファイルを追記保存します。",
                     description_en="Save the processing result json file by appending."),
                dict(opt="stdout_log", type=Options.T_BOOL, default=True, required=False, multi=False, hide=True, choice=[True, False],
                     description_ja="GUIモードでのみ使用可能です。コマンド実行時の標準出力をConsole logに出力します。",
                     description_en="Available only in GUI mode. Outputs standard output during command execution to Console log."),
                dict(opt="capture_stdout", type=Options.T_BOOL, default=True, required=False, multi=False, hide=True, choice=[True, False],
                     description_ja="GUIモードでのみ使用可能です。コマンド実行時の標準出力をキャプチャーし、実行結果画面に表示します。",
                     description_en="Available only in GUI mode. Captures standard output during command execution and displays it on the execution result screen."),
                dict(opt="capture_maxsize", type=Options.T_INT, default=self.DEFAULT_CAPTURE_MAXSIZE, required=False, multi=False, hide=True, choice=None,
                     description_ja="GUIモードでのみ使用可能です。コマンド実行時の標準出力の最大キャプチャーサイズを指定します。",
                     description_en="Available only in GUI mode. Specifies the maximum capture size of standard output when executing commands."),
            ]
        )

    def apprun(self, logger:logging.Logger, args:argparse.Namespace, tm:float, pf:List[Dict[str, float]]=[]) -> Tuple[int, Dict[str, Any], Any]:
        """
        この機能の実行を行います

        Args:
            logger (logging.Logger): ロガー
            args (argparse.Namespace): 引数
            tm (float): 実行開始時間
            pf (List[Dict[str, float]]): 呼出元のパフォーマンス情報

        Returns:
            Tuple[int, Dict[str, Any], Any]: 終了コード, 結果, オブジェクト
        """
        rd = redis.Redis(logger=logger, wsl_name=args.wsl_name, wsl_user=args.wsl_user)
        ret = rd.docker_stop()
        common.print_format(ret, args.format, tm, args.output_json, args.output_json_append, pf=pf)

        return self.RESP_SUCCESS, ret, rd



from pathlib import Path
from setuptools import setup
from setuptools.command.install import install
import io
import platform
import re

def get_info(rel_path):
    fpath = Path(__file__).parent / rel_path
    version = ''
    with io.open(fpath, encoding='utf-8') as fp:
        content = fp.read()
        version = re.search(r"^__version__\s+=\s+'(.*)'", content, re.M).group(1)
    return version

VERSION = get_info('iinfer/version.py')

# ------------------------------------------
# カスタムインストールのロジック (pyproject.tomlで置き換え不可)
# ------------------------------------------
class CustomInstallCommand(install):
    def run(self):
        super().run()
        if platform.system() != 'Linux':
            return
        bashrc = Path.home() / '.bashrc'
        if not bashrc.exists():
            return
        CMD = 'eval "$(register-python-argcomplete iinfer)"'
        with open(bashrc, 'r') as fp:
            for line in fp:
                if line == CMD:
                    return
        with open(bashrc, 'a') as fp:
            fp.write('\n'+CMD)

# setup()関数の呼び出しは、残りの設定をpyproject.tomlから取得するために最小限に抑えます
# dynamic = [...] の設定を有効にするために、setuptoolsのversionは渡しません
setup(
    version=VERSION,  # pyproject.tomlの[project]のversionフィールドを無効化するために渡す
    # setup.cfg/pyproject.tomlと互換性を持たせるため、cmdclassとcmdclassに依存する
    # install_requires/versionを除く最小限の引数のみを保持
    cmdclass={'install': CustomInstallCommand},
)


from pathlib import Path
import os
import sys

if __name__ == "__main__":
    os.chdir(Path(__file__).resolve().parent.parent)
    sys.path.insert(0, "")
    from iinfer.app import app
    exit_code = app.main(webcall=True)
    exit(exit_code)



from cmdbox.app import common, edge_tool, feature
from cmdbox.app.options import Options
from iinfer.app import client
from typing import Dict, Any, Tuple, Union, List
import cv2
import argparse
import logging
import time


class ClientCapture(feature.Feature):
    def get_mode(self) -> Union[str, List[str]]:
        """
        この機能のモードを返します

        Returns:
            Union[str, List[str]]: モード
        """
        return 'client'

    def get_cmd(self):
        """
        この機能のコマンドを返します

        Returns:
            str: コマンド
        """
        return 'capture'
    
    def get_option(self):
        """
        この機能のオプションを返します

        Returns:
            Dict[str, Any]: オプション
        """
        return dict(
            use_redis=self.USE_REDIS_FALSE, nouse_webmode=False,
            description_ja="クライアント側でキャプチャー画像を取得します。",
            description_en="Get a capture image on the client side.",
            test_assert="assert result != ''",
            choice=[
                dict(opt="capture_device", type=Options.T_STR, default="0", required=True, multi=False, hide=False, choice=None,
                     description_ja="キャプチャーディバイスを指定します。 `cv2.VideoCapture` の第一引数に渡される値。",
                     description_en="Specify the capture device. The value passed to the first argument of `cv2.VideoCapture`.",
                     test_true={"jpeg":"0",},
                     test_false={"jpeg":"-1"}),
                dict(opt="image_type", type=Options.T_STR, default="capture", required=True, multi=False, hide=False, choice=['bmp', 'png', 'jpeg', 'capture'],
                     description_ja="出力する画像のタイプを指定します。",
                     description_en="Specify the type of image to output.",
                     test_true={"jpeg":"capture",
                                "bmp":"bmp",
                                "png":"png",
                                "capture":"capture"}),
                dict(opt="capture_frame_width", type=Options.T_INT, default=640, required=False, multi=False, hide=True, choice=None,
                     description_ja="キャプチャーする画像の横px。 `cv2.VideoCapture` オブジェクトの `cv2.CAP_PROP_FRAME_WIDTH` オプションに指定する値。",
                     description_en="Width px of the image to be captured. The value to be specified in the `cv2.CAP_PROP_FRAME_WIDTH` option of the `cv2.VideoCapture` object.",
                     test_true={"jpeg":640},
                     test_false={"jpeg":-10}),
                dict(opt="capture_frame_height", type=Options.T_INT, default=480, required=False, multi=False, hide=True, choice=None,
                     description_ja="キャプチャーする画像の縦px。 `cv2.VideoCapture` オブジェクトの `cv2.CAP_PROP_FRAME_HEIGHT` オプションに指定する値。",
                     description_en="Height px of the image to be captured. The value to be specified in the `cv2.CAP_PROP_FRAME_HEIGHT` option of the `cv2.VideoCapture` object.",
                     test_true={"jpeg":480},
                     test_false={"jpeg":-10}),
                dict(opt="capture_fps", type=Options.T_INT, default=5, required=False, multi=False, hide=True, choice=None,
                     description_ja="キャプチャーする画像のFPS。キャプチャーが指定した値より高速な場合に残り時間分をsleepします。",
                     description_en="FPS of the image to be captured. If the capture is faster than the specified value, sleep for the remaining time.",
                     test_true={"jpeg":5},
                     test_false={"jpeg":-1}),
                dict(opt="capture_count", type=Options.T_INT, default=5, required=False, multi=False, hide=False, choice=None,
                     description_ja="キャプチャーする回数。",
                     description_en="Number of captures.",
                     test_true={"jpeg":3},
                     test_false={"jpeg":-1}),
                dict(opt="output_preview", type=Options.T_BOOL, default=False, required=False, multi=False, hide=False, choice=[True, False],
                     description_ja="推論結果画像を `cv2.imshow` で表示します。",
                     description_en="Display the inference result image with `cv2.imshow`.",
                     test_true={"jpeg":True}),
                dict(opt="output_csv", type=Options.T_FILE, default=None, required=False, multi=False, hide=True, choice=None, fileio="out",
                     description_ja="入力した内容をcsvで保存します。これを指定した場合、標準出力は行いません。",
                     description_en="Saves the input as a csv file. If this is specified, no standard output is performed.",
                     test_true={"jpeg":"capture.csv"}),
                dict(opt="stdout_log", type=Options.T_BOOL, default=True, required=False, multi=False, hide=True, choice=[True, False],
                     description_ja="GUIモードでのみ使用可能です。コマンド実行時の標準出力をConsole logに出力します。",
                     description_en="Available only in GUI mode. Outputs standard output during command execution to Console log."),
                dict(opt="capture_stdout", type=Options.T_BOOL, default=True, required=False, multi=False, hide=True, choice=[True, False],
                     description_ja="GUIモードでのみ使用可能です。コマンド実行時の標準出力をキャプチャーし、実行結果画面に表示します。",
                     description_en="Available only in GUI mode. Captures standard output during command execution and displays it on the execution result screen."),
                dict(opt="capture_maxsize", type=Options.T_INT, default=self.DEFAULT_CAPTURE_MAXSIZE, required=False, multi=False, hide=True, choice=None,
                     description_ja="GUIモードでのみ使用可能です。コマンド実行時の標準出力の最大キャプチャーサイズを指定します。",
                     description_en="Available only in GUI mode. Specifies the maximum capture size of standard output when executing commands."),
            ]
        )


    def apprun(self, logger:logging.Logger, args:argparse.Namespace, tm:float, pf:List[Dict[str, float]]=[]) -> Tuple[int, Dict[str, Any], Any]:
        """
        この機能の実行を行います

        Args:
            logger (logging.Logger): ロガー
            args (argparse.Namespace): 引数
            tm (float): 実行開始時間
            pf (List[Dict[str, float]]): 呼出元のパフォーマンス情報

        Returns:
            Tuple[int, Dict[str, Any], Any]: 終了コード, 結果, オブジェクト
        """
        cl = client.Client(logger, redis_host=self.default_host, redis_port=self.default_port, redis_password=self.default_pass, svname='server')
        count = 0
        append = False
        ret = None
        try:
            for t,b64,h,w,c,fn in cl.capture(capture_device=args.capture_device, image_type=args.image_type,
                                capture_frame_width=args.capture_frame_width, capture_frame_height=args.capture_frame_height,
                                capture_fps=args.capture_fps, output_preview=args.output_preview):
                ret = f"{t},"+b64+f",{h},{w},{c},{fn}"
                if args.output_csv is not None:
                    with open(args.output_csv, 'a' if append else 'w', encoding="utf-8") as f:
                        print(ret, file=f)
                        append = True
                else: common.print_format(ret, False, tm, None, False, pf=pf)
                tm = time.perf_counter()
                count += 1
                if args.capture_count > 0 and count >= args.capture_count:
                    break
        finally:
            try:
                cv2.destroyWindow('preview')
            except:
                pass
        return self.RESP_SUCCESS, ret, cl

    def edgerun(self, opt:Dict[str, Any], tool:edge_tool.Tool, logger:logging.Logger, timeout:int, prevres:Any=None):
        """
        この機能のエッジ側の実行を行います

        Args:
            opt (Dict[str, Any]): オプション
            tool (edge.Tool): 通知関数などedge側のUI操作を行うためのクラス
            logger (logging.Logger): ロガー
            timeout (int): タイムアウト時間
            prevres (Any): 前コマンドの結果。pipeline実行の実行結果を参照する時に使用します。

        Yields:
            Tuple[int, Dict[str, Any], Any]: 終了コード, 結果
        """
        cl = client.Client(logger, redis_host=self.default_host, redis_port=self.default_port, redis_password=self.default_pass, svname='server')
        count = 0
        try:
            for t,b64,h,w,c,fn in cl.capture(capture_device=opt['capture_device'], image_type=opt['image_type'],
                                capture_frame_width=opt['capture_frame_width'], capture_frame_height=opt['capture_frame_height'],
                                capture_fps=opt['capture_fps'], output_preview=opt['output_preview']):
                prevres = f"{t},"+b64+f",{h},{w},{c},{fn}"
                count += 1
                if opt['capture_count'] > 0 and count >= opt['capture_count']:
                    break
                yield 0, prevres
            yield 1, prevres
        finally:
            try:
                cv2.destroyWindow('preview')
            except:
                pass



from cmdbox.app import common, feature
from cmdbox.app.options import Options
from iinfer.app import postprocess
from typing import Dict, Any, Tuple, Union, List
import cv2
import logging
import time
import sys


class PostprocessFeature(feature.Feature):

    def _to_proc(self, f, proc:postprocess.Postprocess, timeout, format, tm,
                    output_json, output_json_append, output_image_file=None, output_csv=None, pf:List[Dict[str, float]]=[]):
        try:
            ret = None
            for line in f:
                line = line.rstrip()
                if line == "":
                    continue
                try:
                    if proc.logger.level == logging.DEBUG:
                        line_str = common.to_str(line, slise=100)
                        proc.logger.debug(f"app.main: args.mode={self.get_mode()}, args.cmd={self.get_cmd()}, proc={proc}, line={line_str}")
                    ret = proc.postprocess(line, output_image_file=output_image_file, timeout=timeout)
                    if output_csv is not None:
                        with open(output_csv, 'a' if output_json_append else 'w', encoding="utf-8") as f:
                            txt = common.print_format(ret, format, tm, output_json, output_json_append, stdout=False, pf=pf)
                            print(txt.strip(), file=f)
                    else: common.print_format(ret, format, tm, output_json, output_json_append, pf=pf)
                except Exception as e:
                    msg = {"warn":f"Invalid input. {e}"}
                    line_str = common.to_str(line, slise=100)
                    proc.logger.error(f"Invalid input.: args.mode={self.get_mode()}, args.cmd={self.get_cmd()}, proc={proc}, line={line_str}", exc_info=True)
                    common.print_format(msg, format, tm, output_json, output_json_append, pf=pf)
                    ret = msg
                tm = time.perf_counter()
                output_json_append = True
            return ret
        finally:
            try:
                cv2.destroyWindow('preview')
            except:
                pass

    def _exec_proc(self, input_file, stdin, proc:postprocess.Postprocess, timeout, format, tm,
                    output_json, output_json_append, output_image_file=None, output_csv=None, pf:List[Dict[str, float]]=[]):
        if input_file is not None:
            with open(input_file, 'r', encoding="UTF-8") as f:
                ret = self._to_proc(f, proc, timeout, format, tm, output_json, output_json_append,
                                output_image_file=output_image_file, output_csv=output_csv, pf=pf)
        elif stdin:
            ret = self._to_proc(sys.stdin, proc, timeout, format, tm, output_json, output_json_append,
                            output_image_file=output_image_file, output_csv=output_csv, pf=pf)
        else:
            msg = {"warn":f"Image file or stdin is empty."}
            common.print_format(msg, format, tm, output_json, output_json_append, pf=pf)
            return self.RESP_WARN, msg
        return self.RESP_SUCCESS, ret
